/* Test of matrix operations */

#include <stdlib.h>

#include <testlib/testlib.h>

#include "matvec/macros.h"

#include "macros.h"

#define MAT4X4(matrix) MATVEC_MAT4X4_AS_ARRAY(matrix,4)
#define MAT3X3(matrix) MATVEC_MAT3X3_AS_ARRAY(matrix,3)

/* quiet -Werror=missing-prototypes */
void test_MATVEC_MAT3X3_ASSIGN( void );
void test_MATVEC_MAT4X4_ASSIGN( void );
void test_MATVEC_MAT3X3_SET( void );
void test_MATVEC_MAT4X4_SET( void );
void test_MATVEC_MAT3X3_IDENTITY( void );
void test_MATVEC_MAT4X4_IDENTITY( void );
void test_MATVEC_MAT3X3_TRANSPOSE( void );
void test_MATVEC_MAT4X4_TRANSPOSE( void );
void test_MATVEC_MAT3X3_MULT( void );
void test_MATVEC_MAT4X4_MULT( void );


void test_MATVEC_MAT3X3_ASSIGN( void )
{
  def_MAT3X3_AS_ARRAY;
  MATTYPE matb[9];

  MATVEC_MAT3X3_ASSIGN(MAT3X3(matb), MAT3X3(mat));
  TEST( "MATVEC_MAT3X3_ASSIGN", T3X3AR_EQ( mat, matb ), 1 );
}


void test_MATVEC_MAT4X4_ASSIGN( void )
{
  def_MAT4X4_AS_ARRAY;
  MATTYPE matb[16];

  MATVEC_MAT4X4_ASSIGN(MAT4X4(matb), MAT4X4(mat));

  TEST( "MATVEC_MAT4X4_ASSIGN", T4X4AR_EQ( mat, matb ), 1 );
}


void test_MATVEC_MAT3X3_SET( void )
{
  MATTYPE mata[9] = { 1, 1, 1,
		      1, 1, 1,
		      1, 1, 1 };
  MATTYPE matb[9];

  MATVEC_MAT3X3_SET( MAT3X3(matb), 1 );
  TEST( "MATVEC_MAT3X3_SET", T3X3AR_EQ( mata, matb ), 1 );
}


void test_MATVEC_MAT4X4_SET( void )
{
  MATTYPE mata[16] = { 1, 1, 1, 1,
		       1, 1, 1, 1,
		       1, 1, 1, 1,
		       1, 1, 1, 1 };
  MATTYPE matb[16];

  MATVEC_MAT3X3_SET( MAT3X3(matb), 1 );
  TEST( "MATVEC_MAT3X3_SET", T3X3AR_EQ( mata, matb ), 1 );
}


void test_MATVEC_MAT3X3_IDENTITY( void )
{
  MATTYPE mata[9] = { 1, 0, 0,
		      0, 1, 0,
		      0, 0, 1 };
  MATTYPE matb[9];

  MATVEC_MAT3X3_IDENTITY( MAT3X3(matb) );
  TEST( "MATVEC_MAT3X3_IDENTITY", T3X3AR_EQ( mata, matb ), 1 );
}


void test_MATVEC_MAT4X4_IDENTITY( void )
{
  MATTYPE mata[16] = { 1, 0, 0, 0,
		       0, 1, 0, 0,
		       0, 0, 1, 0,
		       0, 0, 0, 1 };
  MATTYPE matb[16];

  MATVEC_MAT4X4_IDENTITY( MAT4X4(matb) );
  TEST( "MATVEC_MAT4X4_IDENTITY", T4X4AR_EQ( mata, matb ), 1 );
}


void test_MATVEC_MAT3X3_TRANSPOSE( void )
{
  def_MAT3X3_AS_ARRAY;
  MATTYPE mata[9];
  MATTYPE matb[9] = { 00, 10, 20,
		      01, 11, 21,
		      02, 12, 22  };

  MATVEC_MAT3X3_TRANSPOSE(MAT3X3(mata), MAT3X3(mat));

  TEST( "MATVEC_MAT3X3_TRANSPOSE", T3X3AR_EQ( mata, matb ), 1 );
}


void test_MATVEC_MAT4X4_TRANSPOSE( void )
{
  def_MAT4X4_AS_ARRAY;
  MATTYPE mata[16];
  MATTYPE matb[16] = { 00, 10, 20, 30,
		       01, 11, 21, 31,
		       02, 12, 22, 32,
		       03, 13, 23, 33 };

  MATVEC_MAT4X4_TRANSPOSE(MAT4X4(mata), MAT4X4(mat) );

  TEST( "MATVEC_MAT4X4_TRANSPOSE", T4X4AR_EQ( mata, matb ), 1 );
}

void test_MATVEC_MAT3X3_MULT( void )
{
  MATTYPE a[9] = { 3,  4,  2,
		   2,  3, -1,
		   1, -1,  2 };

  MATTYPE b[9] = { 1, -2, -4,
		   0, -1,  2,
		   6, -3,  9 };

  MATTYPE exp[9] = { 15, -16,  14,
		     -4, - 4, -11, 
		     13, - 7,  12 };

  MATTYPE d[9];


  MATVEC_MAT3X3_MULT(MAT3X3(d), MAT3X3(a), MAT3X3(b));

  TEST( "MATVEC_MAT3X3_MULT", T3X3AR_EQ( d, exp ), 1 );
}

void test_MATVEC_MAT4X4_MULT( void )
{
  MATTYPE a[16] = { 3,  4,  2, 1,
		    2,  3, -1, 1, 
		    1, -1,  2, 1,
		    1, -1,  2, 1  };

  MATTYPE b[16] = { 1, -2, -4, 2,
		    0, -1,  2, 0,
		    6, -3,  9, 12,
		    6,  0,  9, 12 };

  MATTYPE exp[16] = { 21, -16,  23, 42,
		       2, - 4, - 2, 4,
		      19, - 7,  21, 38,
		      19, - 7,  21, 38  };


  MATTYPE d[16];

  MATVEC_MAT4X4_MULT(MAT4X4(d), MAT4X4(a), MAT4X4(b));

  TEST( "MATVEC_MAT4X4_MULT", T4X4AR_EQ( d, exp ), 1 );
}


int main( void )
{
  START( "Matrix Operations" );

  test_MATVEC_MAT3X3_ASSIGN();
  test_MATVEC_MAT4X4_ASSIGN();
  test_MATVEC_MAT3X3_SET();
  test_MATVEC_MAT4X4_SET();
  test_MATVEC_MAT3X3_IDENTITY();
  test_MATVEC_MAT4X4_IDENTITY();
  test_MATVEC_MAT3X3_TRANSPOSE();
  test_MATVEC_MAT4X4_TRANSPOSE();

  test_MATVEC_MAT3X3_MULT();
  test_MATVEC_MAT4X4_MULT();

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
